
;;;
;;; KENDALL
;;; util.scm -- miscellaneous utility and i/o functions
;;; $Id: util.scm 52 2005-09-11 12:45:39Z avatar $
;;;

; (define (filter p s)
;   (cond ((null? s) '())
; 	((p (car s))
; 	 (cons (car s)
; 	       (filter p (cdr s))))
; 	(else (filter p (cdr s)))))

(define (curry p f)
  (lambda (s)
    (p f s)))

(define (kendall-error msg)
  (with-output-to-port (current-error-port)
    (lambda ()
      (print msg)
      (newline)))
  (exit 1))

;; Infinity-ready versions of standard arithmetic functions
(define (inf-plus a b)
  (if (or (eq? a 'inf)
	  (eq? b 'inf))
      'inf
      (+ a b)))

(define (inf-minus a b)
  (if (or (eq? a 'inf)
	  (eq? b 'inf))
      'inf
      (- a b)))

(define (inf-lt a b)
  (cond ((eq? a 'inf) #f)
	((eq? b 'inf) #t)
	(else
	 (< a b))))

(define (inf-min as)
  (let ([l (filter (lambda (x)
		     (not (eq? x 'inf)))
		   as)])
    (if (null? l)
	'inf
	(apply min l))))

;; Build rule lists from input lines. When called with nil argument, return list of rules.
;; E.g. 0#s0#s1#2#foo becomes (0 "s0" "s1" 2 "foo")
(define parse-input-line
  (let ([rules '()])
    (lambda (line)
      (if (not (null? line))
	  (let ((rule (string-split line ":")))
	    (set! rules (cons
			 (list
			  (string->number (number rule))
			  (lhs rule)
			  (rhs rule)
			  (string->number (cost rule))
			  (action rule))
			 rules)))
	  (reverse rules)))))

;; File to store BURS tables in
(define output-file-name "kendall.out")

;; Read file output-file-name and return tables needed by BURS automata
(define (read-kendall-tables)
  (if (not (file-exists? output-file-name))
      (kedall-error "No output file found -- run kendall first!")
      (let* ([p (open-input-file output-file-name)]
	     [rules (read p)]
	     [states (read p)]
	     [leaf-states (read p)]
	     [op-map (read p)]
	     [trans-table (read p)]
	     [rules-table (read p)])
	(close-input-port p)
	(values
	 rules states leaf-states op-map trans-table rules-table))))

;; Write tables to output file in ready-for-scheme-reader format
(define (print-kendall-tables)
  (let ([p (open-output-file output-file-name)])
    (write rules p)
    (write states p)
    (write leaf-states p)
    (write op-map p)
    (write transition-table p)
    (write rules-table p)
    (close-output-port p)))

;; Write tables to output file in unixy format
(define (print-kendall-tables-unixy)
  (let* ([p (open-output-file output-file-name)]
	 [print-label (lambda (s)
			(newline p)
			(display s p)
			(newline p))])
    (print-label "RULES")
    (print-lists rules p)
    (print-label "RTABLE")
    (print-table rules-table p)
    (print-label "OP-MAP")
    (print-table op-map p)
    (print-label "LEAF-STATES")
    (print-alist leaf-states p)
    (print-label "STATES")
    (print-states states p)
    (print-label "TTABLE")
    (print-ttable transition-table p)
    (close-output-port p)))

(define (print-list list p)
  (letrec ([loop (lambda (l)
		   (if (null? l)
		       (newline p)
		       (begin
			 (format p "~a~0,15@T" (car l))
			 (loop (cdr l)))))])
    (loop list)))

(define (print-lists lols p)
  (let ([fun (lambda (l)
	       (print-list l p))])
  (for-each fun lols)))

(define (print-table table p)
  (letrec ([cidx (map car (cdr (car table)))]
	   [loop (lambda (l)
		   (if (null? l)
		       (newline p)
		       (begin
			 (format p "~a~0,10@T" (car l))
			 (loop (cdr l)))))]
	   [print-rows (lambda (row)
			 (let ([lbl (car row)]
			       [vals (map cdr (cdr row))])
			   (format p "~a~0,10@T" lbl)
			   (loop vals)))])
    (format p "~10,10@T")
    (loop cidx)
    (for-each print-rows table)))

(define (print-alist a p)
  (for-each (lambda (e)
	      (format p "~a~0,15@T~a~%" (car e) (cdr e)))
	    a))

(define (print-states table p)
  (letrec ([cols (map car (car (car table)))]
	   [funroll (lambda (entry)
		      (let ([lbl (cdr entry)]
			    [vals (map cdr (car entry))]) ; rule-cost pairs
			(format p "~a~0,10@T" lbl)
			(let baz ([l vals])
			  (if (null? l)
			      (newline p)
			      (begin
				(format p "~a/~a~0,10@T"
					(rcp-rule (car l))
					(rcp-cost (car l)))
				(baz (cdr l)))))))])
    (format p "~10,10@T")
    (let quux ([l cols])
      (if (null? l)
	  (newline p)
	  (begin
	    (format p "~a~0,10@T" (car l))
	    (quux (cdr l)))))
    (for-each funroll table)))

(define (print-ttable a p)
  (letrec ([fun (lambda (l)
		  (let ([pairs (cdr l)]
			[lbl (car l)])
		    (format p "~a~0,10@T" lbl)
		    (let loop ([x pairs])
		      (if (null? x)
			  (newline p)
			  (begin
			    (format p "~a->~a~0,10@T" (car (car x)) (cdr (car x)))
			    (loop (cdr x)))))))])
    (for-each fun a)))

;;; EOF
